@@ -343,6 +343,90 @@ static void sCommand_copy_deeply_stack(sCommand* dest, sCommand* src)
     }
 }
 
+static void sCommand_copy_deeply_malloc(sCommand* dest, sCommand* src)
+{
+    dest->mKind = src->mKind;
+
+    dest->mArgs = MALLOC(sizeof(char*)*src->mArgsSize);
+    dest->mArgsEnv = MALLOC(sizeof(int)*src->mArgsSize);
+    dest->mArgsNum = src->mArgsNum;
+    dest->mArgsSize = src->mArgsSize;
+
+    int i;
+    for(i=0; i<src->mArgsNum; i++) {
+        dest->mArgs[i] = STRDUP(src->mArgs[i]);
+        dest->mArgsEnv[i] = src->mArgsEnv[i];
+    }
+    dest->mArgs[i] = NULL;
+
+    dest->mEnvs = MALLOC(sizeof(sEnv)*src->mEnvsSize);
+    for(i=0; i<src->mEnvsNum; i++) {
+        dest->mEnvs[i].mKind = src->mEnvs[i].mKind;
+
+        if(src->mEnvs[i].mKind == kEnv) {
+            dest->mEnvs[i].mName = STRDUP(src->mEnvs[i].mName);
+            dest->mEnvs[i].mKey = STRDUP(src->mEnvs[i].mKey);
+            dest->mEnvs[i].mKeyEnv = src->mEnvs[i].mKeyEnv;
+            dest->mEnvs[i].mDoubleDollar = src->mEnvs[i].mDoubleDollar;
+        }
+        else {
+            dest->mEnvs[i].mBlock = block_clone_malloc(src->mEnvs[i].mBlock);
+        }
+    }
+    dest->mEnvsNum = src->mEnvsNum;
+    dest->mEnvsSize = src->mEnvsSize;
+
+/*
+    dest->mArgsRuntime = MALLOC(sizeof(char*)*src->mArgsSizeRuntime);
+    dest->mArgsNumRuntime = src->mArgsNumRuntime;
+    dest->mArgsSizeRuntime = src->mArgsSizeRuntime;
+
+    for(i=0; i<src->mArgsNumRuntime; i++) {
+        dest->mArgsRuntime[i] = STRDUP(src->mArgsRuntime[i]);
+    }
+    dest->mArgsRuntime[i] = NULL;
+*/
+    dest->mArgsRuntime = MALLOC(sizeof(char*)*8);
+    dest->mArgsSizeRuntime = 8;
+    dest->mArgsNumRuntime = 0;
+
+    dest->mBlocks = MALLOC(sizeof(sObject*)*src->mBlocksSize);
+    dest->mBlocksNum = src->mBlocksNum;
+    dest->mBlocksSize = src->mBlocksSize;
+
+    for(i=0; i<src->mBlocksNum; i++) {
+        dest->mBlocks[i] = block_clone_malloc(src->mBlocks[i]);
+    }
+
+    dest->mRedirectsFileNames = MALLOC(sizeof(char*)*src->mRedirectsSize);
+    for(i=0; i<src->mRedirectsNum; i++) {
+        dest->mRedirectsFileNames[i] = STRDUP(src->mRedirectsFileNames[i]);
+    }
+    dest->mRedirects = MALLOC(sizeof(int)*src->mRedirectsSize);
+    for(i=0; i<src->mRedirectsNum; i++) {
+        dest->mRedirects[i] = src->mRedirects[i];
+    }
+    dest->mRedirectsSize = src->mRedirectsSize;
+    dest->mRedirectsNum = src->mRedirectsNum;
+
+/*
+    memcpy(dest->mOptions, src->mOptions, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+
+    for(i=0; i<XYZSH_OPTION_MAX; i++) {
+        if(src->mOptions[i].mKey) { dest->mOptions[i].mKey = STRDUP(src->mOptions[i].mKey); }
+        if(src->mOptions[i].mArg) { dest->mOptions[i].mArg = STRDUP(src->mOptions[i].mArg); }
+    }
+*/
+    memset(dest->mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+
+    if(src->mMessagesNum > 0) {
+        for(i=0; i<src->mMessagesNum; i++) {
+            dest->mMessages[i] = STRDUP(src->mMessages[i]);
+        }
+        dest->mMessagesNum = src->mMessagesNum;
+    }
+}
+
 void sCommand_delete(sCommand* self)
 {
     int i;
@@ -478,6 +562,21 @@ static void sStatment_copy_deeply_stack(sStatment* dest, sStatment* source)
     dest->mFlags = source->mFlags;
 }
 
+static void sStatment_copy_deeply_malloc(sStatment* dest, sStatment* source)
+{
+    dest->mCommandsNum = source->mCommandsNum;
+
+    int i;
+    for(i=0; i<source->mCommandsNum; i++) {
+        sCommand_copy_deeply_malloc(dest->mCommands + i, source->mCommands + i);
+    }
+
+    dest->mFName = STRDUP(source->mFName);
+    dest->mLine = source->mLine;
+
+    dest->mFlags = source->mFlags;
+}
+
 void sStatment_delete(sStatment* self)
 {
     int i;
@@ -588,6 +687,33 @@ sObject* block_clone_gc(sObject* source, int type, BOOL user_object)
     return self;
 }
 
+sObject* block_clone_malloc(sObject* source)
+{
+    sObject* self = MALLOC(sizeof(sObject));
+
+    const int statments_size = SBLOCK(source).mStatmentsSize;
+    
+    SBLOCK(self).mStatments = MALLOC(sizeof(sStatment)*statments_size);
+    memset(SBLOCK(self).mStatments, 0, sizeof(sStatment)*statments_size);
+    SBLOCK(self).mStatmentsNum = SBLOCK(source).mStatmentsNum;
+    SBLOCK(self).mStatmentsSize = statments_size;
+    if(SBLOCK(source).mSource) {
+        SBLOCK(self).mSource = STRDUP(SBLOCK(source).mSource);
+    }
+    else {
+        SBLOCK(self).mSource = NULL;
+    }
+    SBLOCK(self).mCompletionFlags = SBLOCK(source).mCompletionFlags;
+
+    /// copy contents of all statments
+    int i;
+    for(i=0; i<SBLOCK(source).mStatmentsNum; i++) {
+        sStatment_copy_deeply_malloc(SBLOCK(self).mStatments + i, SBLOCK(source).mStatments + i);
+    }
+
+    return self;
+}
+
 sObject* block_clone_stack(sObject* source, int type)
 {
     sObject* self = stack_get_free_object(type);
@@ -1027,7 +1153,8 @@ sObject* fun_new_from_gc(sObject* parent, BOOL user_object)
    SFUN(self).mArgBlocks = NULL;
 
    SFUN(self).mRunInfo = NULL;
-
+   SFUN(self).mOptions = MALLOC(sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+   memset(SFUN(self).mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
 
    return self;
 }
@@ -1048,6 +1175,8 @@ sObject* fun_new_from_stack(sObject* parent)
    SFUN(self).mArgBlocks = NULL;
 
    SFUN(self).mRunInfo = NULL;
+   SFUN(self).mOptions = MALLOC(sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+   memset(SFUN(self).mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
 
    return self;
 }
@@ -1062,6 +1191,8 @@ sObject* fun_clone_from_stack_block_gc(sObject* block, BOOL user_object, sObject
    SFUN(self).mArgBlocks = NULL;
 
    SFUN(self).mRunInfo = NULL;
+   SFUN(self).mOptions = MALLOC(sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+   memset(SFUN(self).mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
 
    return self;
 }
@@ -1076,6 +1207,8 @@ sObject* fun_clone_from_stack_block_stack(sObject* block, sObject* parent)
    SFUN(self).mArgBlocks = NULL;
 
    SFUN(self).mRunInfo = NULL;
+   SFUN(self).mOptions = MALLOC(sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+   memset(SFUN(self).mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
 
    return self;
 }
@@ -1094,6 +1227,7 @@ void fun_delete_gc(sObject* self)
         if(SFUN(self).mOptions[i].mKey) { FREE(SFUN(self).mOptions[i].mKey); }
         if(SFUN(self).mOptions[i].mArg) { FREE(SFUN(self).mOptions[i].mArg); }
     }
+    FREE(SFUN(self).mOptions);
 }
 
 void fun_delete_stack(sObject* self)
@@ -1110,6 +1244,7 @@ void fun_delete_stack(sObject* self)
         if(SFUN(self).mOptions[i].mKey) { FREE(SFUN(self).mOptions[i].mKey); }
         if(SFUN(self).mOptions[i].mArg) { FREE(SFUN(self).mOptions[i].mArg); }
     }
+    FREE(SFUN(self).mOptions);
 }
 
 int fun_gc_children_mark(sObject* self)
@@ -1234,6 +1369,8 @@ sObject* class_new_from_gc(sObject* parent, BOOL user_object)
 
    SFUN(self).mLocalObjects = NULL;
    SFUN(self).mArgBlocks = NULL;
+   SFUN(self).mOptions = MALLOC(sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+   memset(SFUN(self).mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
 
    return self;
 }
@@ -1246,6 +1383,8 @@ sObject* class_clone_from_stack_block_gc(sObject* block, BOOL user_object, sObje
 
    SFUN(self).mLocalObjects = NULL;
    SFUN(self).mArgBlocks = NULL;
+   SFUN(self).mOptions = MALLOC(sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+   memset(SFUN(self).mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
 
    return self;
 }
@@ -1258,6 +1397,8 @@ sObject* class_clone_from_stack_block_stack(sObject* block, sObject* parent)
 
    SFUN(self).mLocalObjects = NULL;
    SFUN(self).mArgBlocks = NULL;
+   SFUN(self).mOptions = MALLOC(sizeof(option_hash_it)*XYZSH_OPTION_MAX);
+   memset(SFUN(self).mOptions, 0, sizeof(option_hash_it)*XYZSH_OPTION_MAX);
 
    return self;
 }
@@ -1276,6 +1417,7 @@ void class_delete_gc(sObject* self)
         if(SFUN(self).mOptions[i].mKey) { FREE(SFUN(self).mOptions[i].mKey); }
         if(SFUN(self).mOptions[i].mArg) { FREE(SFUN(self).mOptions[i].mArg); }
     }
+    FREE(SFUN(self).mOptions);
 }
 
 BOOL class_put_option_with_argument(sObject* self, MANAGED char* key)
@@ -1340,6 +1482,7 @@ void class_delete_stack(sObject* self)
         if(SFUN(self).mOptions[i].mKey) { FREE(SFUN(self).mOptions[i].mKey); }
         if(SFUN(self).mOptions[i].mArg) { FREE(SFUN(self).mOptions[i].mArg); }
     }
+    FREE(SFUN(self).mOptions);
 }
 
 void sCommand_sweep_runtime_info(sCommand* self)
