@@ -237,7 +237,7 @@ BOOL cmd_stackinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     char buf[BUFSIZ];
 
-    snprintf(buf, BUFSIZ, "slot size %d\npool size %d\n", stack_slot_size(), stack_page_size());
+    snprintf(buf, BUFSIZ, "slot size %d\npool size %d\nall object number %d\n", stack_slot_size(), stack_page_size(), stack_slot_size()*stack_page_size());
 
     if(!fd_write(nextout, buf)) {
         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -343,7 +343,7 @@ BOOL cmd_gcinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sCommand* command = runinfo->mCommand;
     char buf[BUFSIZ];
 
-    snprintf(buf, BUFSIZ, "free objects %d\nused objects %d\nall object slot %d\nslot size %d\npool size %d\n", gc_free_objects_num(), gc_used_objects_num(), gc_slot_size()*gc_pool_size(), gc_slot_size(), gc_pool_size());
+    snprintf(buf, BUFSIZ, "free objects %d\nused objects %d\nall object number %d\nslot size %d\npool size %d\n", gc_free_objects_num(), gc_used_objects_num(), gc_slot_size()*gc_pool_size(), gc_slot_size(), gc_pool_size());
 
     if(!fd_write(nextout, buf)) {
         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -683,7 +683,7 @@ static BOOL xyzsh_seek_external_programs(char* path, sObject* sys, char* sname,
         char buf[BUFSIZ];
         snprintf(buf, BUFSIZ, "%s is invalid path", path);
         err_msg(buf, sname, sline, "rehash");
-        return FALSE;
+        //return FALSE;
     }
 
     return TRUE;
