@@ -154,7 +154,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int rcode = 0;
             sObject* block = BLOCK_NEW_STACK();
             int sline = 1;
-            if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL)) {
+            if(!parse(SFD(nextin).fdbuf.mBuf, "def", &sline, block, NULL)) {
                 err_msg("parser error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 stack_end_stack();
                 return FALSE;
@@ -166,7 +166,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             stack_end_stack();
             
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -222,7 +222,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int rcode = 0;
             sObject* block = BLOCK_NEW_STACK();
             int sline = 1;
-            if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL)) {
+            if(!parse(SFD(nextin).fdbuf.mBuf, "def", &sline, block, NULL)) {
                 err_msg("parser error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 stack_end_stack();
                 return FALSE;
@@ -234,7 +234,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             stack_end_stack();
             
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -465,10 +465,10 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).mLines);
+            int max = vector_size(SFD(nextin).fdbuf.mLines);
             for(i=1; i<command->mArgsNumRuntime; i++) {
                 if(i-1 < max) {
-                    sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i-1), TRUE);
+                    sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i-1), TRUE);
                     string_chomp(new_var);
                     hash_put(object, command->mArgsRuntime[i], new_var);
                 }
@@ -479,7 +479,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             if(sCommand_option_item(command, "-shift")) {
                 for(;i-1<max; i++) {
-                    if(!fd_write(nextout, vector_item(SFD(nextin).mLines, i-1))) {
+                    if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, i-1))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -487,7 +487,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -688,9 +688,9 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* object = runinfo->mCurrentObject;
 
             int i;
-            int max = vector_size(SFD(nextin).mLines);
+            int max = vector_size(SFD(nextin).fdbuf.mLines);
             for(i=0; i<max; i++) {
-                char* line = vector_item(SFD(nextin).mLines, i);
+                char* line = vector_item(SFD(nextin).fdbuf.mLines, i);
                 void* mem = (void*)(unsigned long)strtoll(line, NULL, 16);
                 if(mem == NULL) break;
 
@@ -715,7 +715,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -727,11 +727,11 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 sObject* object = runinfo->mCurrentObject;
 
                 int i;
-                int max = vector_size(SFD(nextin).mLines);
+                int max = vector_size(SFD(nextin).fdbuf.mLines);
                 BOOL shift = sCommand_option_item(command, "-shift");
                 for(i=0; i<max; i++) {
                     if(i+1 < command->mArgsNumRuntime) {
-                        char* line = vector_item(SFD(nextin).mLines, i);
+                        char* line = vector_item(SFD(nextin).fdbuf.mLines, i);
                         void* mem = (void*)(unsigned long)strtoll(line, NULL, 16);
                         int type = memchecker_item(gMemChecker, mem);
 
@@ -744,7 +744,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         }
                     }
                     else if(shift) {
-                        if(!fd_write(nextout, vector_item(SFD(nextin).mLines, i))) {
+                        if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, i))) {
                             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
@@ -755,7 +755,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
                 }
 
-                if(*(SFD(nextin).mBuf) == 0) {
+                if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                     runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
                 }
                 else {
@@ -920,17 +920,17 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(append > vector_size(ary)) append = vector_size(ary);
 
             int i;
-            int max = vector_size(SFD(nextin).mLines);
+            int max = vector_size(SFD(nextin).fdbuf.mLines);
             void** appended_ary = MALLOC(sizeof(sObject*)*max);
             for(i=0; i<max; i++) {
-                sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
+                sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i), FALSE);
                 string_chomp(new_var);
                 appended_ary[i] = new_var;
             }
             vector_mass_insert(ary, append, appended_ary, max);
             FREE(appended_ary);
             
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -947,16 +947,16 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).mLines);
+            int max = vector_size(SFD(nextin).fdbuf.mLines);
             sObject* new_ary = VECTOR_NEW_GC(16, TRUE);
             hash_put(object, command->mArgsRuntime[1], new_ary);
             for(i=0; i<max; i++) {
-                sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
+                sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i), FALSE);
                 string_chomp(new_var);
                 vector_add(new_ary, new_var);
             }
             
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -1115,21 +1115,21 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).mLines);
+            int max = vector_size(SFD(nextin).fdbuf.mLines);
             sObject* key = STRING_NEW_STACK("");
             for(i=0; i<max; i++) {
                 if((i % 2) == 0) {
-                    string_put(key, vector_item(SFD(nextin).mLines, i));
+                    string_put(key, vector_item(SFD(nextin).fdbuf.mLines, i));
                     string_chomp(key);
                 }
                 else {
-                    sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
+                    sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i), FALSE);
                     string_chomp(new_var);
                     hash_put(hash, string_c_str(key), new_var);
                 }
             }
                 
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -1146,23 +1146,23 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).mLines);
+            int max = vector_size(SFD(nextin).fdbuf.mLines);
             sObject* new_hash = HASH_NEW_GC(16, TRUE);
             hash_put(object, command->mArgsRuntime[1], new_hash);
             sObject* key = STRING_NEW_STACK("");
             for(i=0; i<max; i++) {
                 if((i % 2) == 0) {
-                    string_put(key, vector_item(SFD(nextin).mLines, i));
+                    string_put(key, vector_item(SFD(nextin).fdbuf.mLines, i));
                     string_chomp(key);
                 }
                 else {
-                    sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, i), FALSE);
+                    sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i), FALSE);
                     string_chomp(new_var);
                     hash_put(new_hash, string_c_str(key), new_var);
                 }
             }
                 
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
@@ -1227,10 +1227,10 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         if(command->mArgsNumRuntime > 1) {
             int i;
-            int max = vector_size(SFD(nextin).mLines);
+            int max = vector_size(SFD(nextin).fdbuf.mLines);
             for(i=1; i<command->mArgsNumRuntime; i++) {
                 if(i-1 < max) {
-                    sObject* new_var = STRING_NEW_STACK(vector_item(SFD(nextin).mLines, i-1));
+                    sObject* new_var = STRING_NEW_STACK(vector_item(SFD(nextin).fdbuf.mLines, i-1));
                     string_chomp(new_var);
                     setenv(command->mArgsRuntime[i], string_c_str(new_var), 1);
                 }
@@ -1241,7 +1241,7 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             if(sCommand_option_item(command, "-shift")) {
                 for(;i-1<max; i++) {
-                    if(!fd_write(nextout, vector_item(SFD(nextin).mLines, i-1))) {
+                    if(!fd_write(nextout, vector_item(SFD(nextin).fdbuf.mLines, i-1))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
@@ -1249,7 +1249,7 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            if(*(SFD(nextin).mBuf) == 0) {
+            if(*(SFD(nextin).fdbuf.mBuf) == 0) {
                 runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
             }
             else {
