@@ -7,11 +7,16 @@ static void version()
     printf("xyzsh version %s\n", getenv("XYZSH_VERSION"));
 }
 
+static void main_xyzsh_job_done(int job_num, char* job_title)
+{
+    printf("%d: %s done.\n", job_num+1, job_title);
+}
+
 int main(int argc, char** argv)
 {
     CHECKML_BEGIN(FALSE);
 
-    setenv("XYZSH_VERSION", "1.0.1", 1);
+    setenv("XYZSH_VERSION", "1.0.3", 1);
     setenv("XYZSH_DATAROOTDIR", DATAROOTDIR, 1);
 
     srandom((unsigned)time(NULL));
@@ -36,17 +41,23 @@ int main(int argc, char** argv)
         }
     }
 
+
     if(optc) {
-        xyzsh_opt_c(optc, argv, argc, no_runtime_script);
+        xyzsh_init(kATOptC, no_runtime_script);
+        xyzsh_opt_c(optc, argv, argc);
     }
     else if(file) {
-        xyzsh_load_file(file, argv, argc, no_runtime_script);
+        xyzsh_init(kATOptC, no_runtime_script);
+        xyzsh_load_file(file, argv, argc);
     }
     else {
+        xyzsh_init(kATConsoleApp, no_runtime_script);
         setenv("XYZSH_RUNNING_AS_INTRACTIVE_MODE", "1", 1);
-        xyzsh_readline_interface(argv, argc, no_runtime_script);
+        xyzsh_readline_interface("", -1, argv, argc);
     }
 
+    xyzsh_final();
+
     CHECKML_END();
 
     return 0;
