@@ -756,6 +756,32 @@ BOOL cmd_readline_delete_text(sObject* nextin, sObject* nextout, sRunInfo* runin
     return TRUE;
 }
 
+BOOL cmd_readline_read_history(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mArgsNumRuntime == 2) {
+        char* fname = command->mArgsRuntime[1];
+        read_history(fname);
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL cmd_readline_write_history(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mArgsNumRuntime == 2) {
+        char* fname = command->mArgsRuntime[1];
+        write_history(fname);
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
 static int readline_macro(int count, int key)
 {
     stack_start_stack();
@@ -857,51 +883,12 @@ char* readline_filename_completion_null_generator(const char* a, int b)
     return NULL;
 }
 
-char* readline_quote_func (char* text, int match_type, char* quote_pointer)
-{
-    char* p = text;
-    char* result = malloc(strlen(text) * 2 + 1);
-    char* p2 = result;
-    while(*p) {
-        if(*p == ' ') {
-            *p2++ = '\\';
-            *p2++ = *p++;
-        }
-        else {
-            *p2++ = *p++;
-        }
-    }
-
-    *p2 = 0;
-
-    return result;
-}
-
-char* readline_dequote_func (char* text, int quote_char)
-{
-    char* p = text;
-    char* result = malloc(strlen(text) * 2 + 1);
-    char* p2 = result;
-    while(*p) {
-        if(*p == '\\') {
-            p++;
-            *p2++ = *p++;
-        }
-        else {
-            *p2++ = *p++;
-        }
-    }
-    *p2 = 0;
-
-    return result;
-}
 
 enum { COMPLETE_DQUOTE,COMPLETE_SQUOTE,COMPLETE_BSQUOTE };
 #define completion_quoting_style COMPLETE_BSQUOTE
 
 static BOOL shell_cmd;
 
-/* Quote STRING using double quotes.  Return a new string. */
 static char *
 double_quote (char *string)
 {
@@ -935,8 +922,6 @@ double_quote (char *string)
   return (result);
 }
 
-/* Return a new string which is the single-quoted version of STRING.
-   Used by alias and trap, among others. */
 static char *
 single_quote (char *string)
 {
@@ -953,9 +938,9 @@ single_quote (char *string)
 
       if (c == '\'')
 	{
-	  *r++ = '\\';	/* insert escaped single quote */
+	  *r++ = '\\';	// insert escaped single quote
 	  *r++ = '\'';
-	  *r++ = '\'';	/* start new quoted string */
+	  *r++ = '\'';	// start new quoted string
 	}
     }
 
@@ -968,8 +953,6 @@ single_quote (char *string)
 static BOOL quote_glob;
 static BOOL inhibit_tilde;
 
-/* Quote special characters in STRING using backslashes.  Return a new
-   string. */
 static char *
 backslash_quote (char *string)
 {
@@ -983,31 +966,30 @@ backslash_quote (char *string)
       switch (c)
 	{
  	case '(': case ')':
- 	case '{': case '}':			/* reserved words */
+ 	case '{': case '}':			// reserved words
  	case '^':
- 	case '$': case '`':			/* expansion chars */
+ 	case '$': case '`':			// expansion chars
 	  if(!shell_cmd)
 	    goto def;
- 	case '*': case '[': case '?': case ']':	/* globbing chars */
+ 	case '*': case '[': case '?': case ']':	//globbing chars
 	  if(!shell_cmd && !quote_glob)
 	    goto def;
-	case ' ': case '\t': case '\n':		/* IFS white space */
-	case '"': case '\'': case '\\':		/* quoting chars */
-	case '|': case '&': case ';':		/* shell metacharacters */
+	case ' ': case '\t': case '\n':		// IFS white space
+	case '"': case '\'': case '\\':		// quoting chars
+	case '|': case '&': case ';':		// shell metacharacters
 	case '<': case '>': case '!':
 	  *r++ = '\\';
 	  *r++ = c;
 	  break;
-	case '~':				/* tilde expansion */
+	case '~':				// tilde expansion
 	  if (s == string && inhibit_tilde)
 	    *r++ = '.', *r++ = '/';
 	  goto def;
-	case '#':				/* comment char */
+	case '#':				// comment char
 	  if(!shell_cmd)
 	    goto def;
 	  if (s == string)
 	    *r++ = '\\';
-	  /* FALLTHROUGH */
 	default: def:
 	  *r++ = c;
 	  break;
@@ -1018,9 +1000,6 @@ backslash_quote (char *string)
   return (result);
 }
 
-/* Quote characters that the readline completion code would treat as
-   word break characters with backslashes.  Pass backslash-quoted
-   characters through without examination. */
 static char *
 quote_word_break_chars (char *text)
 {
@@ -1031,7 +1010,6 @@ quote_word_break_chars (char *text)
   ret = (char*)malloc ((2 * l) + 1);
   for (s = text, r = ret; *s; s++)
     {
-      /* Pass backslash-quoted characters through, including the backslash. */
       if (*s == '\\')
 	{
 	  *r++ = '\\';
@@ -1040,8 +1018,6 @@ quote_word_break_chars (char *text)
 	    break;
 	  continue;
 	}
-      /* OK, we have an unquoted character.  Check its presence in
-	 rl_completer_word_break_characters. */
       if (strchr (rl_completer_word_break_characters, *s))
         *r++ = '\\';
       *r++ = *s;
@@ -1058,13 +1034,6 @@ bash_quote_filename (char *s, int rtype, char *qcp)
 
   rtext = (char *)NULL;
 
-  /* If RTYPE == MULT_MATCH, it means that there is
-     more than one match.  In this case, we do not add
-     the closing quote or attempt to perform tilde
-     expansion.  If RTYPE == SINGLE_MATCH, we try
-     to perform tilde expansion, because single and double
-     quotes inhibit tilde expansion by the shell. */
-
   mtext = s;
 #if 0
   if (mtext[0] == '~' && rtype == SINGLE_MATCH)
@@ -1072,8 +1041,6 @@ bash_quote_filename (char *s, int rtype, char *qcp)
 #endif
 
   cs = completion_quoting_style;
-  /* Might need to modify the default completion style based on *qcp,
-     since it's set to any user-provided opening quote. */
   if (*qcp == '"')
     cs = COMPLETE_DQUOTE;
   else if (*qcp == '\'')
@@ -1107,8 +1074,6 @@ bash_quote_filename (char *s, int rtype, char *qcp)
   if (mtext != s)
     free (mtext);
 
-  /* We may need to quote additional characters: those that readline treats
-     as word breaks that are not quoted by backslash_quote. */
   if (rtext && cs == COMPLETE_BSQUOTE)
     {
       mtext = quote_word_break_chars (rtext);
@@ -1116,13 +1081,10 @@ bash_quote_filename (char *s, int rtype, char *qcp)
       rtext = mtext;
     }
 
-  /* Leave the opening quote intact.  The readline completion code takes
-     care of avoiding doubled opening quotes. */
   rlen = strlen (rtext);
   ret = (char*)malloc (rlen + 1);
   strcpy (ret, rtext);
 
-  /* If there are multiple matches, cut off the closing quote. */
   if (rtype == MULT_MATCH && cs != COMPLETE_BSQUOTE)
     ret[rlen - 1] = '\0';
   free (rtext);
@@ -1141,7 +1103,6 @@ bash_dequote_filename (const char *text, int quote_char)
   ret = (char*)malloc (l + 1);
   for (quoted = quote_char, p = text, r = ret; p && *p; p++)
     {
-      /* Allow backslash-quoted characters to pass through unscathed. */
       if (*p == '\\')
 	{
 	  *r++ = *++p;
@@ -1149,13 +1110,11 @@ bash_dequote_filename (const char *text, int quote_char)
 	    break;
 	  continue;
 	}
-      /* Close quote. */
       if (quoted && *p == quoted)
         {
           quoted = 0;
           continue;
         }
-      /* Open quote. */
       if (quoted == 0 && (*p == '\'' || *p == '"'))
         {
           quoted = *p;
@@ -1174,14 +1133,17 @@ void xyzsh_readline_init(BOOL runtime_script)
     rl_completer_quote_characters = "\"'";
     rl_completer_word_break_characters = " \t\n\"'|!&;()$%<";
     rl_completion_append_character= ' ';
-    rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
     rl_filename_quote_characters = " \t\n\"'|!&;()$%<";
     rl_filename_quoting_function = bash_quote_filename;
     rl_filename_dequoting_function = (rl_dequote_func_t*)bash_dequote_filename;
 
-    //rl_comrl_filename_quote_characters = " \t\n\\'\"()$&|>";
-    //rl_completer_quote_characters = " \t\n\\'\"()$&|>";
-    //rl_basic_quote_characters = " \t\n\"'|!&;()$";
+    rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
+
+/*
+    rl_comrl_filename_quote_characters = " \t\n\\'\"()$&|>";
+    rl_completer_quote_characters = " \t\n\\'\"()$&|>";
+    rl_basic_quote_characters = " \t\n\"'|!&;()$";
+*/
 
     rl_bind_key('x'-'a'+1, readline_macro);
 }
