@@ -370,13 +370,15 @@ sObject* job_new_from_gc(char* name, pid_t pgroup, struct termios tty)
 
     SJOB(self).mName = STRDUP(name);
     SJOB(self).mPGroup = pgroup;
-    SJOB(self).mTty = tty;
+    SJOB(self).mTty = MALLOC(sizeof(struct termios));
+    memcpy(SJOB(self).mTty, &tty, sizeof(struct termios));
 
     vector_add(gJobs, self);
 }
 
 void job_delete_gc(sObject* self)
 {
+    FREE(SJOB(self).mTty);
     if(SJOB(self).mName) FREE(SJOB(self).mName);
 }
 
@@ -1450,7 +1452,7 @@ BOOL forground_job(int num)
     if(num>=0 && num < vector_size(gJobs)) {
         sObject* job = vector_item(gJobs, num);
 
-        tcsetattr(STDIN_FILENO, TCSANOW, &SJOB(job).mTty);
+        tcsetattr(STDIN_FILENO, TCSANOW, SJOB(job).mTty);
         if(tcsetpgrp(0, SJOB(job).mPGroup) < 0) {
             perror("tcsetpgrp(fg)");
             return FALSE;
@@ -1497,7 +1499,7 @@ BOOL forground_job(int num)
         }
         /// stopped from signal ///
         else if(WIFSTOPPED(status)) {
-            tcgetattr(STDIN_FILENO, &SJOB(job).mTty);
+            tcgetattr(STDIN_FILENO, SJOB(job).mTty);
 
             /// forground xyzsh ///
             if(tcsetpgrp(0, getpgid(0)) < 0) {
