@@ -15,7 +15,7 @@ volatile BOOL gxyzshSigCont;
 
 sObject* gDirStack;
 
-static void xyzsh_read_rc_core(char* path)
+void xyzsh_read_rc_core(char* path)
 {
     sRunInfo runinfo;
     char** argv = NULL;
@@ -149,10 +149,13 @@ void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
     if(!no_runtime_script) {
         xyzsh_read_rc();
     }
+
+    readline_read_history();
 }
 
 void xyzsh_final()
 {
+    readline_write_history();
 #if defined(HAVE_MIGEMO_H)
     migemo_final();
 #endif
@@ -524,8 +527,4 @@ char* xstrncat(char* des, char* str, int size)
     return strncat(des, str, size-1);
 }
 
-int xyzsh_job_num()
-{
-    return vector_size(gJobs);
-}
 
