@@ -148,7 +148,7 @@ typedef struct {
 } fd_obj;
 
 typedef struct {
-    char mPath[PATH_MAX];
+    char* mPath;
 } external_prog_obj;
 
 struct sStatment;
@@ -271,28 +271,28 @@ typedef struct                // a function object can be treated as block_obj
     struct _sObject* mLocalObjects;
     struct _sObject* mArgBlocks;
 
-    option_hash_it mOptions[XYZSH_OPTION_MAX];
+    option_hash_it* mOptions;
     struct _sRunInfo* mRunInfo;
 } fun_obj;
 
-typedef BOOL (*fNativeFun)(struct _sObject*, struct _sObject*, struct _sRunInfo*);
+typedef BOOL (*fXyzshNativeFun)(struct _sObject*, struct _sObject*, struct _sRunInfo*);
 
 typedef struct 
 {
-    fNativeFun mNativeFun;
+    fXyzshNativeFun mNativeFun;
     struct _sObject* mParent;
 
-    option_hash_it mOptions[XYZSH_OPTION_MAX];
+    option_hash_it* mOptions;
 } nfun_obj;
 
 typedef struct {
     char* mName;                // the job name
     pid_t mPGroup;              // process group id
-    struct termios mTty;        // a terminal status
+    struct termios* mTty;       // a terminal status
     BOOL mSuspended;
 } job_obj;
 
-typedef struct {                // a user object can be treated like a hash_obj
+typedef struct {          // a user object can be treated like a hash_obj
    hash_it** mTable;
    int mTableSize;
 
@@ -401,14 +401,14 @@ sObject* stack_get_free_object(int kind);
 int stack_slot_size();
 int stack_page_size();
 
-#include "debug.h"
-#include "hash.h"
-#include "kanji.h"
-#include "string.h"
-#include "vector.h"
-#include "list.h"
-#include "block.h"
-#include "curses.h"
+#include <xyzsh/debug.h>
+#include <xyzsh/hash.h>
+#include <xyzsh/kanji.h>
+#include <xyzsh/string.h>
+#include <xyzsh/vector.h>
+#include <xyzsh/list.h>
+#include <xyzsh/block.h>
+#include <xyzsh/curses.h>
 
 sObject* extobj_new_from_gc(void* object, fExtObjMarkFun mark_fun, fExtObjFreeFun free_fun, BOOL user_object);
 #define EXTOBJ_NEW_GC(o, o2, o3, o4) extobj_new_from_gc(o, o2, o3, o4)
@@ -426,7 +426,7 @@ void uobject_delete_stack(sObject* self);
 int uobject_gc_children_mark(sObject* self);
 
 sObject* external_new_from_gc(char* path, BOOL user_object);
-sObject* external_delete_gc();
+void external_delete_gc();
 
 #define EXTERNAL_NEW_GC(o, o2) external_new_from_gc(o, o2)
 
@@ -460,6 +460,8 @@ BOOL xyzsh_rehash(char* sname, int sline);
 BOOL cmd_migemo_match(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 #endif
 BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_read_history(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_write_history(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_readline_point_move(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_readline_clear_screen(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -622,7 +624,8 @@ void xyzsh_set_signal_optc();
 void sigchld_block(int block);
 
 extern void (*xyzsh_set_signal_other)();
-extern void (*xyzsh_job_done)(int job_num, char* job_title);
+typedef void (*fXyzshJobDone)(int job_num, char* job_title);
+extern fXyzshJobDone xyzsh_job_done;
 
 BOOL forground_job(int num);
 BOOL background_job(int num);
@@ -665,9 +668,9 @@ extern sObject* gStackFrames;
 BOOL bufsiz_write(int fd, char* buf);
 
 BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc);
-void xyzsh_load_file(char* fname, char** argv, int argc, BOOL no_runtime_script);
-void xyzsh_opt_c(char* cmd, char** argv, int argc, BOOL no_runtime_script);
-void xyzsh_readline_interface(char** argv, int argc, BOOL no_runtime_script);
+void xyzsh_load_file(char* fname, char** argv, int argc);
+void xyzsh_opt_c(char* cmd, char** argv, int argc);
+void xyzsh_readline_interface(char* cmdline, int cursor, char** argv, int argc);
 
 void fd_split(sObject* self, enum eLineField lf);
 
@@ -685,7 +688,7 @@ int memchecker_item(sObject* self, void* key);
 
 int object_gc_children_mark(sObject* self);
 
-sObject* nfun_new_from_gc(fNativeFun fun, sObject* parent, BOOL user_object);
+sObject* nfun_new_from_gc(fXyzshNativeFun fun, sObject* parent, BOOL user_object);
 #define NFUN_NEW_GC(o, o2, o3) nfun_new_from_gc(o, o2, o3)
 int nfun_gc_children_mark(sObject* self);
 void nfun_delete_gc(sObject* self);
@@ -752,7 +755,19 @@ void clear_matching_info_variable();
 #define REDIRECT_APPEND 2
 #define REDIRECT_OUT 3
 
-extern int xyzsh_job_num();
+void xyzsh_shell(char* cmdline, char* source_name, sObject* nextin, sObject* nextout, BOOL add_history, fXyzshJobDone xyzsh_job_done_, char** argv, int argc);
+void xyzsh_readline_interface_onetime(char* cmdline, int cursor_point, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_);
+char* xyzsh_job_title(int n);
+int xyzsh_job_num();
+int xyzsh_run(sObject* block, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv);
+
+#include <stdarg.h>
+
+void xyzsh_add_inner_command(sObject* object, char* name, fXyzshNativeFun command, int arg_num, ...);
+
+void readline_write_history();
+void readline_read_history();
+void xyzsh_read_rc_core(char* path);
 
 #endif
 
